@@ -13,7 +13,7 @@ from pstats import Stats
 from jinja2 import Template
 
 
-__version__ = '0.1.2'
+__version__ = '0.1.3'
 __author__  = 'Hideo Hattori <syobosyobo@gmail.com>'
 __licence__ = 'New BSD License'
 
@@ -26,13 +26,7 @@ XHTML_HEADER_TEMPLATE = u"""\
 <head>
 
 <style type="text/css">
-body {
-    font-size: 120%;
-    line-height: 1.5;
-    margin: 0px;
-    padding:0px;
-}
-
+body { font-size: 120%; line-height: 1.5; margin: 0px; padding:0px; }
 div.header {
     font-family: Georgia;
     background-color: #eae0d5;
@@ -46,27 +40,11 @@ span.header-title {
     margin-left: 1em;
     float: left;
 }
-span.header-time {
-    text-align: right;
-    width: 50%;
-    margin-right: 1em;
-}
-div.content {
-    clear: both;
-    font-family: Arial, Verdana;
-}
-table.prof {
-    border-collapse: collapse;
-    border: 1px #cccccc solid;
-}
-td.prof {
-    border-collapse: collapse;
-    border: 1px #cccccc solid;
-}
-span.tdmg {
-  margin-left: 5px;
-  margin-right: 5px;
-}
+span.header-time { text-align: right; width: 50%; margin-right: 1em; }
+div.content { clear: both; font-family: Arial, Verdana; }
+table.prof { border-collapse: collapse; border: 1px #cccccc solid; }
+td.prof { border-collapse: collapse; border: 1px #cccccc solid; }
+span.tdmg { margin-left: 5px; margin-right: 5px; }
 pre.tdms {
   margin-left: 5px;
   margin-right: 5px;
@@ -86,11 +64,7 @@ td.profline {
     font-size: 75%;
     padding:0px;
 }
-th.procheader {
-    align: center;
-    background: #a294c8;
-}
-
+th.procheader { align: center; background: #a294c8; }
 #sidebar {
     position: fixed;
     padding-top: 30px;
@@ -100,13 +74,7 @@ th.procheader {
     top  : 0;
     width: 160px;
 }
-
-#content {
-    margin-left : 1em;
-    margin-right: 1em;
-    text-align: left;
-}
-
+#content { margin-left : 1em; margin-right: 1em; text-align: left; }
 #footer {
     clear     : both;
     text-align: center;
@@ -116,14 +84,7 @@ th.procheader {
     margin-bottom: -1em;
     background-color: #eae0d5;
 }
-
-div.profilelinetable {
-    margin-top: 1em;
-}
-
-h1.blog-title {
-    margin-top : 0px;
-}
+div.profilelinetable { margin-top: 1em; }
 
 </style>
 """
@@ -143,11 +104,19 @@ XHTML_BODY_TEMPLATE = u"""\
     </div>
 
     <div id="content">
-    <p>Total Time     : <span style="font-weight:bold">{{ profdata['totaltime'] }}</span> CPU second<br/>
-       Total Func Call: <span style="font-weight:bold">{{ profdata['totalcalls'] }}</span> function calls</p>
+    <p>Total Time     : <span style="font-weight:bold"
+                        >{{ profdata['totaltime'] }}</span> CPU second<br/>
+       Total Func Call: <span style="font-weight:bold"
+                        >{{ profdata['totalcalls'] }}</span> function calls</p>
     <table class="prof">
         <tr>
-            <th class="procheader" align="center" colspan="6">Top {{ profdata['data']|length }} functions data</th>
+            <th class="procheader" align="center" colspan="6"
+            >Top {{ profdata['data']|length }} functions data (<a href="
+            {% if thisname == 'index.html' %}
+                index-all.html">all function)</th>
+            {% else %}
+                index.html">top 20 function</a>)</th>
+            {% endif %}
         </tr>
         <tr align="right">
             <th class="prof"><span class="tdmg" title="for the number of calls">ncalls</span></th>
@@ -172,7 +141,12 @@ XHTML_BODY_TEMPLATE = u"""\
             {% if item['func'][0] == '~' %}
                 <td class="prof"><span class="tdmg">{{ item['func'][2]|escape }}</span></td>
             {% else %}
-                <td class="prof"><span class="tdmg">{{ item['func'][0]|escape }} : 
+                <td class="prof"><span class="tdmg"
+                {% if proftype == 'line' %}
+                    ><a href="{{ item['linelink'] }}">{{ item['func'][0]|escape }}</a> : 
+                {% else %}
+                    >{{ item['func'][0]|escape }}</a> : 
+                {% endif %}
                 {{ item['func'][1] }} ({{ item['func'][2]|escape }})</span></td>
             {% endif %}
             </tr>
@@ -181,13 +155,32 @@ XHTML_BODY_TEMPLATE = u"""\
 """
 
 XHTML_LINEBODY_TEMPLATE = u"""\
+<title>{{ title }}</title>
+</head>
+
+<body>
+    <div class="header">
+        <span class="header-title">Profile Report</span>
+        <span class="header-time">Profile Time on {{ proftime }}</span><br/>
+        <span class="header-time">Reported Time on {{ reporttime }}</span>
+    </div>
+
+    <div id="sidebar">
+    </div>
+
+    <div id="content">
+    <p>Total Time     : <span style="font-weight:bold"
+                        >{{ profdata['totaltime'] }}</span> CPU second<br/>
+       Total Func Call: <span style="font-weight:bold"
+                        >{{ profdata['totalcalls'] }}</span> function calls</p>
     <div class="profilelinetable">
     <table class="prof">
     <tr>
     <th class="procheader" align="center" colspan="3">{{ filename }}</th>
     </tr>
     <tr align="right">
-    <th class="prof" align="center"><span class="tdmg">&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;time&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</span></th>
+    <th class="prof" align="center"><span class="tdmg"
+     >&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;time&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</span></th>
     <th class="prof" align="center"><span class="tdmg">num</span></th>
     <th class="prof" align="center"><span class="tdmg">code</span></th>
     </tr>
@@ -196,13 +189,17 @@ XHTML_LINEBODY_TEMPLATE = u"""\
         {% if item['sec'] == 'None' %}
             <td class="profline" align="right"><span class="tdms"></span></td>
         {% else %}
-            <td class="profline" align="right" style="background-color:#ffffff;"><span class="tdms">{{ item['sec'] }} sec</span></td>
+            <td class="profline" align="right" style="background-color:#ffffff;"
+             ><span class="tdms">{{ item['sec'] }} sec</span></td>
         {% endif %}
-        <td class="profline" align="right"><span class="tdms" style="color:#999999;">{{ loop.index }}</span></td>
+        <td class="profline" align="right"><span class="tdms" style="color:#999999;"
+                                            >{{ loop.index }}</span></td>
         {% if item['sec'] == 'None' %}
-            <td class="profline" align="left"><pre class="tdms">{{ item['line']|escape }}</pre></td>
+            <td class="profline" align="left"><pre class="tdms"
+                                               >{{ item['line']|escape }}</pre></td>
         {% else %}
-            <td class="profline" align="left"><pre class="tdms" style="font-weight:bold;">{{ item['line']|escape }}</pre></td>
+            <td class="profline" align="left"><pre class="tdms" style="font-weight:bold;"
+                                               >{{ item['line']|escape }}</pre></td>
         {% endif %}
         </tr>
     {% endfor %}
@@ -234,7 +231,7 @@ def check_hotshot(dump):
     return signature in dump
 
 def mapping_table(target, all):
-    level_list = [all*0.01, all*0.03, all*0.08, all*0.12, all*0.20,]
+    level_list = [all*0.01, all*0.03, all*0.08, all*0.12, all*0.20]
     for level in range(len(level_list)):
         if target < level_list[level]:
             break
@@ -245,6 +242,9 @@ class Converter:
     """output to HTML from profile data"""
 
     def __init__(self, filename):
+        self.outfile = None
+        self.output_dir = 'html'
+        self.output_htmlfile = 'index.html'
         self.proftype = None
         dump = open(filename).read()
         if check_hotshot(dump[:20]):
@@ -259,13 +259,14 @@ class Converter:
             self.tmpl = Template(XHTML_BODY_TEMPLATE)
         self.filename = filename
         self.proftime = time.ctime(os.stat(filename).st_mtime)
+        self.reporttime = time.ctime()
         self.outputtype = 'html'
         self.functions_number = 20
         self.levelmap = {
-            0: '#ccffcc',   ## (like) light cyan
-            1: '#66ff99',   ## (like) light cyan
-            2: '#ffcc33',   ## (like) lime
-            3: '#ff6666',   ## yellow
+            0: '#ccffcc',   ## yellow green (light)
+            1: '#66ff99',   ## yellow green
+            2: '#ffcc33',   ## orange
+            3: '#ff6666',   ## pink
             4: '#ff3333',   ## red
         }
 
@@ -289,7 +290,11 @@ class Converter:
                 result.append({'sec':profinfo[num], 'line':line})
             else:
                 result.append({'sec':'None', 'line':line})
-        return self.tmplline.render(filename=filename,
+        titletext = "pyprof2html - %s" % filename
+        return self.tmplline.render(title=titletext,
+                                    proftime=self.proftime,
+                                    reporttime=self.reporttime,
+                                    filename=filename,
                                     profdata=result,
                                     codec=codec)
 
@@ -322,15 +327,28 @@ class Converter:
                 if info != {}:
                     filecnt += 1
                     if self.outputtype == 'html':
-                        print >> self.outfile, self._printhtml_onesource(source_list[filecnt-1],
-                                                        info)
+                        output_filename = "%s/%s.html" % (self.output_dir,
+                                 os.path.basename(source_list[filecnt-1]))
+                        outputfile = open(output_filename, 'w')
+                        print >> outputfile, XHTML_HEADER_TEMPLATE
+                        print >> outputfile, self._printhtml_onesource(
+                                                  source_list[filecnt-1],
+                                                  info)
+                        print >> outputfile, XHTML_FOOTER_TEMPLATE
+                        outputfile.close()
                     else:
-                        print self._print_onesource(source_list[filecnt-1], info)
+                        print self._print_onesource(source_list[filecnt-1],
+                                                     info)
                 info = {}
                 source_list.append(filename)
             info[prof[1]] = prof[2]
         if self.outputtype == 'html':
-            print >> self.outfile, self._printhtml_onesource(filename, info)
+            output_htmlfile = "%s.html" % os.path.basename(source_list[filecnt-1])
+            outputfile = open("%s/%s" % (self.output_dir, output_htmlfile), 'w')
+            print >> outputfile, XHTML_HEADER_TEMPLATE
+            print >> outputfile, self._printhtml_onesource(filename, info)
+            print >> outputfile, XHTML_FOOTER_TEMPLATE
+            outputfile.close()
         else:
             print self._print_onesource(filename, info)
 
@@ -345,14 +363,6 @@ class Converter:
         proflist.sort()
         return proflist
 
-    def _hookraw(self):
-        if self.proftype == 'line':
-            proflist = self._analyze_profline()
-            self._print_source(proflist)
-        else:
-            self.prof.sort_stats('time', 'calls')
-            self.prof.print_stats()
-
     def _analyzed_prof(self):
         self.prof.sort_stats('time', 'calls')
         null = open('/dev/null', 'w')   # darty hack (order output block)
@@ -382,14 +392,15 @@ class Converter:
                 cumpercall = "%8.4lf" % float(stat[3]/stat[0])
             except ZeroDivisionError:
                 cumpercall = "0.0000"
-            ncallslevel = self.levelmap[mapping_table(ncalls,  self.prof.total_calls)]
-            tottlevel   = self.levelmap[mapping_table(stat[2], self.prof.total_tt)]
-            totclevel   = self.levelmap[mapping_table(float(totpercall), self.prof.total_tt)]
-            #cumtlevel   = self.levelmap[mapping_table(stat[3], self.prof.total_tt)]
-            #cumclevel   = self.levelmap[mapping_table(float(cumpercall), self.prof.total_tt)]
-            #totclevel   = self.levelmap[0]
+            ncallslevel = self.levelmap[mapping_table(ncalls,
+                                                      self.prof.total_calls)]
+            tottlevel   = self.levelmap[mapping_table(stat[2],
+                                                      self.prof.total_tt)]
+            totclevel   = self.levelmap[mapping_table(float(totpercall),
+                                                      self.prof.total_tt)]
             cumtlevel   = self.levelmap[0]
             cumclevel   = self.levelmap[0]
+            linelink    = "%s.html" % os.path.basename(func[0])
             data = {
                 'ncalls'        : ncalls,
                 'tottime'       : tottime,
@@ -402,6 +413,7 @@ class Converter:
                 'totcalllevel'  : totclevel,
                 'cumcalllevel'  : cumclevel,
                 'func'          : func,
+                'linelink'      : linelink,
             }
             datalist.append(data)
         profdata = {
@@ -411,17 +423,35 @@ class Converter:
         }
         return profdata
 
+    def _hookraw(self):
+        """hook rawtext print"""
+        if self.proftype == 'line':
+            proflist = self._analyze_profline()
+            self._print_source(proflist)
+        else:
+            self.prof.sort_stats('time', 'calls')
+            self.prof.print_stats()
+
     def _hookhtml(self):
+        """hook html print"""
+        if not os.path.exists(self.output_dir):
+            os.makedirs(self.output_dir)
+        filepath = "%s/%s" % (self.output_dir, self.output_htmlfile)
+        self.outfile = open(filepath, 'w')
         profdata = self._analyzed_prof()
+        titletext = "pyprof2html - %s" % self.filename
         print >> self.outfile, XHTML_HEADER_TEMPLATE
-        print >> self.outfile, self.tmpl.render(title="pyprof2html - %s" % self.filename,
-                               proftime=self.proftime,
-                               reporttime=time.ctime(),
-                               profdata=profdata)
+        print >> self.outfile, self.tmpl.render(title=titletext,
+                                                proftime=self.proftime,
+                                                reporttime=self.reporttime,
+                                                profdata=profdata,
+                                                thisname=self.output_htmlfile,
+                                                proftype=self.proftype)
         if self.proftype:
             profinfo = self._analyze_profline()
             self._print_source(profinfo)
         print >> self.outfile, XHTML_FOOTER_TEMPLATE
+        self.outfile.close()
 
     def printout(self, filetype='html'):
         """print to html or text."""
@@ -430,10 +460,6 @@ class Converter:
         if filetype == 'raw':
             hookpoint = self._hookraw
         else:
-            if not os.path.exists(self.output_dir):
-                os.makedirs(self.output_dir)
-            filepath = self.output_dir + '/index.html'
-            self.outfile = open(filepath, 'w')
             hookpoint = self._hookhtml
         hookpoint()
 
@@ -470,6 +496,10 @@ def main():
         outmode = 'html'
     p2h.output_dir = opts.output_dir
     p2h.printout(outmode)
+    if outmode == 'html':
+        p2h.functions_number = 99999
+        p2h.output_htmlfile = 'index-all.html'
+        p2h.printout(outmode)
 
 
 if __name__ == '__main__':
